import novotools.utils as tools
from functools import partial


# DocxApi
def docxapi_test():
    """
    test for DocxApi class
    """
    docxapi = tools.DocxApi()

    docxapi.set_font(en_style='Times New Roman',zh_style='微软雅黑')

    title_size = docxapi.get_pt_by_size("一号")
    title = docxapi.add_paras("Word操作测试")
    title = docxapi.align_center(title)

    docxapi.set_para_font_size(title,title_size)
    pic = docxapi.add_picture("./Python.png",2.5,2.5)
    pic = docxapi.align_center(pic)

    second_title_size = docxapi.get_pt_by_size("二号")
    second_title = docxapi.add_paras("段落示例")
    second_title = docxapi.set_para_font_size(second_title,second_title_size)
    docxapi.add_paras("在我所有的程序错误中，80%是语法错误，剩下20%里，80%是简单的逻辑错误，\
    在剩下4%里，80%是指针错误，只有余下的0.8%才是困难的问题。")

    table_list = [
        ["标题","内容"],
        ["DocxApi","使用python操作word文档"]
    ]
    second_title = docxapi.add_paras("表格示例")
    second_title = docxapi.set_para_font_size(second_title,second_title_size)

    docxapi.add_table(table_list)
    ofile = "docxapi_test.docx"
    docxapi.save(ofile = ofile)
    print(f"save docx to {ofile}")


def imap_test():
    """
    test for IMAPEMAIL class
    """
    imap = tools.IMAPEmail(
        user = "medical-sc-info@novogene.com",
        password = "HblM70kQjVb]tYL",
        receive_host = 'imap.exmail.qq.com',
        receive_port = 993,
    )
    
    imap.list_box()
    imap.search_email(
        search_key = "X101SC21072336-Z01-J004",
        box = "&UXZO1mWHTvZZOQ-/AutoPipeline",
        search_type="ALL",
        download_attach= False
    )

def send_email_test():
    """
    send email by send_email
    """
    send_email = partial(tools.send_email,
        **{
        "user" : "medical-sc-info@novogene.com",
        "password" : "HblM70kQjVb]tYL",
        "send_host" : 'smtp.exmail.qq.com',
        "send_port" : 465,
        }    
    )
    
    recipients = ["chenming@novogene.com"]
    subject = "send email test"
    text = "This is a test email for send_email function with attachment"
    files = ["docxapi_test.docx"]
    
    send_email(recipients=recipients,
               subject=subject,
               text=text,
               files=files)
    
def time_test():
    """
    TimeParser test
    """
    import datetime as dt
    
    print(tools.TimeParser.transfer_to("2021-10-22",to="string"))
    print(tools.TimeParser.transfer_to("2021-10-22 15:30:00",to="string"))
    print(tools.TimeParser.transfer_to("2021-10-22 15:30:00.01",to="string"))
    print(tools.TimeParser.transfer_to("2021-10-22 15:30:00.01",to="timestamp"))
    print(type(tools.TimeParser.transfer_to("2021-10-22 15:30:00.01")))
    print(str(dt.datetime.now()))
    print(type(tools.TimeParser.str_to_timedelta("17:55:18.806000")))
    print(type(tools.TimeParser.str_to_timedelta("1 day, 2:18:31.712400")))
    print(type(tools.TimeParser.str_to_timedelta("2 days, 2:18:31.712400")))
    

def title_parse_test():
    """
    test for TitleParser
    """
    title = "gene\tchromosome\tstart\tend\n"
    line = "Foo\t1\t100\t1000\n"
    tp = tools.TitleParser(title,split="\t")
    line_list = line.strip().split("\t")
    gene = tp.get_field(line_list,"gene")
    chr_idx = tp.get_idx("chromosome")
    have_start = tp.have_title("start")
    print(f"Gene name:{gene}; chromosome idx:{chr_idx}; have_start: {have_start}")

def run_cmd_test():
    cmd_normal = "ls ."
    stdout,stderr = tools.run_cmd(cmd_normal)
    print(f"{cmd_normal}标准输出为 {stdout}; 标准错误为 {stderr}")
    cmd_err = "cd not_exist_dir"
    stdout,stderr = tools.run_cmd(cmd_err)
    print(f"{cmd_err}标准输出为 {stdout}; 标准错误为 {stderr}")

def make_colors_test():
    colors = tools.make_colors()
    print(f"{colors['red']}[Warning] {colors['green']} [pass] {colors['cyan']} [info] {colors['back']} [normal]")

def datasize_convert_test():
    trans_size1 = tools.datasize_convert('1024G','T',return_type="float")
    trans_size2 = tools.datasize_convert('1024KB','B',return_type="int")
    trans_size3 = tools.datasize_convert('1024KB','GB',return_type="str")
    print(f"trans_size1 type :{type(trans_size1)}")
    print(f"trans_size2 type :{type(trans_size2)}")
    print(f"trans_size3 type :{type(trans_size3)}")

def basic_log_test():
    import logging
    logger = tools.basic_log("logger",level=logging.DEBUG)
    
    logger.debug("This message is for debug")
    logger.info("This is a information message")
    logger.warning("This is a warning")
    logger.error("This is a error")
    logger.critical("This is the badest conditon!")

def thread_enhance_test():
    
    from threading import current_thread
    
    @tools.thread_enhance()
    def func():
        print(f"Current threading:{current_thread().name}")
        print("start run ...")
        print(1/0)
        
    def func_test():
        th = func()
        th.start()
        th.join()
        
        if th.error != 0:
            print(th.msg)
        else:
            print("No error in thread")
            
        print("main thread end.")
    
    func_test()
    
if __name__ == "__main__":
    # imap_test()
    # send_email_test()
    # title_parse_test()
    # run_cmd_test()
    # make_colors_test()
    datasize_convert_test()
    basic_log_test()
    thread_enhance_test()


#!/usr/bin/env python
# -*- coding=utf-8 -*-

import os
import argparse
# import codecs
import openpyxl
import gzip
from functools import partial

def mkdir_if_not_exists(path):
    if not os.path.exists(path):
        # print 'make a new dir:', path
        # os.makedirs(path, mode=0o777, exist_ok=False)
        os.makedirs(path)
def safe_open(filename, mode='r'):

    try:
        if mode == 'w':
            dirname = os.path.dirname(filename)
            if dirname and not os.path.exists(dirname):
                os.makedirs(dirname)
        if filename.endswith('.gz'):
            mode += 'b'
            return gzip.open(filename, mode)

        return open(filename, mode)
    except Exception as e:
        print('[error] fail to open file:%s' % filename)
        exit(1)

def get_sheetname(filename,sheetnames=None,sheetname_idx=[0]):

    filename = os.path.basename(filename).split('.')

    if not sheetnames:
        sheetname = ".".join([filename[idx] for idx in sheetname_idx])
    else:
        sheetname = sheetnames.pop(0)

    return sheetname[:31]


def text2excel(outfile, *infiles):
    '''
    infiles can be a string: 'a.xls,b.xls'          ('a.xls,b.xls', )
                  or a list: ['a.xls', 'b.xls']     (['a.xls', 'b.xls'], )
    or many positional args: 'a.xls', 'b.xls'       ('a.xls', 'b.xls')
    '''

    # wb = openpyxl.Workbook(encoding='utf8')
    wb = openpyxl.Workbook()

    if len(infiles) == 1:
        if isinstance(infiles[0], str):
            infile_list = infiles[0].split(',')
        elif isinstance(infiles[0], list):
            infile_list = infiles[0]

    elif len(infiles) >= 2:
        infile_list = list(infiles)

    else:
        exit('error infiles: {}'.format(infiles))

    for infile in infile_list:
        sheetname = get_sheetname(filename=infile)
        print('create sheet:%s' % sheetname)
        sheet = wb.create_sheet(title=sheetname)
        # with codecs.open(infile, mode='r', encoding='gbk', errors='ignore') as f:
        with safe_open(infile) as f:
            for n, line in enumerate(f):
                row = n + 1
                linelist = line.strip().split('\t')
                for m, value in enumerate(linelist):
                    column = m + 1
                    sheet.cell(row=row, column=column, value=value)

    # remove default sheet
    try:
        wb.remove(wb['Sheet'])
    except AttributeError:
        wb.remove_sheet(wb.get_sheet_by_name('Sheet'))  # for the old version

    outdir = os.path.dirname(outfile)
    if outdir:
        mkdir_if_not_exists(outdir)

    wb.save(filename=outfile)

    print('write excel file:%s' % outfile)


if __name__ == "__main__":
    usage = '''usage:
    text2excel.py <outfile> <--sheetname|--sheetname_idx> <infiles>
        outfile   the output excel format file
        infiles   the input text format files, separate by comma
    '''
    parser = argparse.ArgumentParser(
        formatter_class = argparse.RawDescriptionHelpFormatter,
        description = usage,
    )
    parser.add_argument('ofile',help="Output excel file name")
    parser.add_argument('--sheetname','-n',help="alist of sheetname seperated by commas")
    parser.add_argument('--sheetname_idx','-i',default="0",help="you can use part of input filename as\
output sheetname which split by dot,for example 0,1,2 sheetname of excel will use first three part of input filename ")
    parser.add_argument('infile',nargs="+",help="alist of infile required to merge to a excel")
    args = vars(parser.parse_args())
    if args.get('sheetname'):
        tags = args.get("sheetname").split(',')
        get_sheetname = partial(get_sheetname,sheetnames=tags)
    else:
        tags = [int(i) for i in args.get('sheetname_idx').split(',')]
        get_sheetname = partial(get_sheetname,sheetname_idx=tags)

    text2excel(args.get("ofile") ,*args.get("infile"))




__all__ = ["email_opt","logging_opt","process_and_thread","string_format_and_file_opt",'time_opt']

from .email_opt import send_email,IMAPEmail
from .logging_opt import basic_log,multi_logger
from .process_and_thread import multi_process,process_pool,run_cmd,thread_enhance
from .string_format_and_file_opt import make_colors,TitleParser,DocxApi,progress_bar,datasize_convert
from .time_opt import TimeParser



import re
import os
import imaplib
import email
import smtplib
from email.header import Header
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from email.mime.application import MIMEApplication
from email.utils import getaddresses

## email tools
#configure

def send_email(
    recipients,
    subject,
    text,
    user,
    password,
    send_host = 'smtp.exmail.qq.com',
    send_port = 465,
    text_type="plain",
    cc=[],
    files=None,
    ):
    """
    text_type = [plain,html]
    """
    #login
    smtp_obj = smtplib.SMTP_SSL(send_host,send_port)
    smtp_obj.login(user,password)
    # recipients = ['cm.bio@qq.com']
    msg = MIMEMultipart()
    msg['from'] = 'medical-sc-info@novogene.com'
    msg['to'] = ";".join(recipients)
    if cc:
        msg['cc'] = ";".join(cc)
    msg['subject'] = Header(subject)
    content = MIMEText(text,text_type,'utf-8')
    msg.attach(content)
    #添加附件
    if files:
        for f in files:
            with open(f,'rb') as indata:
                part = MIMEApplication(indata.read())
                part.add_header('Content-Disposition', 'attachment', filename=f)
                msg.attach(part)
    
    all_recipients = recipients + cc 
    smtp_obj.sendmail(msg['from'],all_recipients,str(msg))

    print('email send successfully!')

class IMAPEmail():
    """
    """
    def __init__(self,
                user,
                password,
                receive_host = 'imap.exmail.qq.com',
                receive_port = 993,
                basic_dir = os.getcwd()
    ):
        #login email client
        self.client = imaplib.IMAP4_SSL(receive_host,receive_port)
        self.client.login(user,password)
        self._savedir = os.path.join(basic_dir,"load_data_emails")

    def _check(self,tag,data):
        if tag == "OK":
            return True
        elif tag == "NO":
            raise Exception(data)

    def decode_boxname(self,boxname):
        """
        parse boxname 
        """
        decode_name = boxname.replace('&', '+').encode('utf-8').decode('utf-7')
        return decode_name
    
    def encode_boxname(self,boxname):
        encodename = boxname.encode("utf-7").replace('+', '&')
        return encodename
    
    def list_box(self):
        #select a inbox
        tag,data = self.client.list()
        if self._check(tag,data):
            for box in data:
                list_response_pattern = re.compile(
                r'.(?P<flags>.*?). "(?P<delimiter>.*)" "(?P<name>.*)"'
                )
                match = list_response_pattern.match(box.decode("utf-8"))
                flags, _, mailbox_name = match.groups()
                flags = flags.replace("\\","")
                # print(f"{mailbox_name:<20}: {flags}")

                status = self.client.status(
                '"{}"'.format(mailbox_name),
                '(MESSAGES RECENT UIDNEXT UIDVALIDITY UNSEEN)',
                )
                status = status[1][0].decode("utf-8")
                box_re = re.search(r'(?P<boxname>.+) \(.*\)',status)
                trans_boxname = self.decode_boxname(box_re.group('boxname'))
                
                print(f'{trans_boxname}:{status}')
                
    def get_all_msg(self,box,search_type):
        all_msg = []
        tag,data = self.client.select(box)
        if self._check(tag,data):
            tag,data = self.client.search(None,search_type)
            # print(tag,data)
            for num in data[0].split():
                tag,data = self.client.fetch(num,'(RFC822)')
                # print(data[0][1])
                msg = email.message_from_bytes(data[0][1])
                all_msg.append(msg)
        return all_msg
    
    def get_body(self,msg):
        if msg.is_multipart():
            print("It's a mutipart!")
            for part in msg.walk():
                ctype = part.get_content_type()
                cdispo = str(part.get('Content-Disposition'))

                # skip any text/plain (txt) attachments
                if ctype == 'text/plain' and ('attachment' not in cdispo):
                    body = part.get_payload(decode=True).decode("utf-8")  # decode
                    break
        # not multipart - i.e. plain text, no attachments, keeping fingers crossed
        else:
            body = msg.get_payload(decode=True).decode("utf-8")
        return body
    
    def get_subject(self,msg):
        # get subject
        text,enc = email.header.decode_header(msg['subject'])[0]
        print(text,enc)
        subject = text.decode(enc) if enc else text
        print(subject)
        return subject
    
    def get_sender_and_recipients(self,msg):
        # get recipients
        tos = msg.get_all('to',[])
        sender = msg.get_all("from",[])
        ccs = msg.get_all("cc",[])
        resent_tos = msg.get_all('resent-to', [])
        resent_ccs = msg.get_all('resent-cc', [])
        
        return getaddresses(tos + ccs + resent_tos + resent_ccs + sender)

    def search_email(self,
        search_key,
        box,
        search_type="ALL",
        download_attach=False):
        """
        This method is used to search emails according to
        a series of filters like box and emails' type
        ----
        box: which box you want to search
        search_key: which content you want to search, [regular expression]
        search_type: email type like UNSEEN ALL
        """
        re_obj_list = []
        tag,data = self.client.select(box)
        if self._check(tag,data):
            tag,data = self.client.search(None,search_type)
            # print(tag,data)
            for num in data[0].split():
                tag,data = self.client.fetch(num,'(RFC822)')
                # print(data[0][1])
                msg = email.message_from_bytes(data[0][1])

                ### email subject fetch
                text,enc = email.header.decode_header(msg['subject'])[0]
                # print(text,enc)
                subject = text.decode(enc) if enc else text
                if search_key in subject:
                    print(f'find {search_key} in {subject},mark email as Seen.')
                    typ, data = self.client.store(num,'+FLAGS', '(\\Seen)')
                    if download_attach:
                        self.get_email_attachment(msg)
                else:
                    ### It looks like the first return of msg.get_payload() is plain text
                    ### The second return is html format
                    body_text = msg.get_payload()[0].get_payload(decode=True).decode('utf-8')
                    for line in body_text.split("\n"):
                        # print(line)
                        if re.search(search_key,line):
                            print(f'find {search_key} in {subject},mark email as Seen.')
                            print(line)
                            typ, data = self.client.store(num,'+FLAGS', '(\\Seen)')
                            if download_attach:
                                self.get_email_attachment(msg)
                            
                    # elif line.startswith("&nbsp;") or line.startswith("From:"):
                    #     print("search completion!")
                    #     break

                # for payload in msg.get_payload():
                #     print(payload.get_payload(decode=True).decode("utf-8"))
    
    def scan_loaddata_email(self,
        box,
        search_type="UNSEEN"):
        attachments = []
        tag,data = self.client.select(box)
        if self._check(tag,data):
            tag,data = self.client.search(None,search_type)
            print(tag,data)
            for num in data[0].split():
                tag,data = self.client.fetch(num,'(RFC822)')
                # print(data[0][1])
                msg = email.message_from_bytes(data[0][1])
                attachments += self.get_email_attachment(msg)
                typ, data = self.client.store(num,'+FLAGS', '(\\Seen)')
                print(f'Download attachment completed, mark email as Seen.')
                
        print(attachments)
        return attachments    

    def get_email_attachment(self,msg,valid_extensions=[".xlsx"]):
        """
        下载邮件中的附件
        """
        attachments = []
        for part in msg.walk():
            if part.get_content_maintype() == 'multipart':
                continue
            if part.get('Content-Disposition') is None:
                continue
            fileName = part.get_filename()

            # 如果文件名为纯数字、字母时不需要解码，否则需要解码
            try:
                fileName = email.header.decode_header(fileName)[0][0].decode(email.header.decode_header(fileName)[0][1])
            except:
                pass

            # 只获取指定拓展名的附件
            extension = os.path.splitext(os.path.split(fileName)[1])[1]
            if valid_extensions:
                if extension not in valid_extensions:
                    continue

            # 如果获取到了文件，则将文件保存在指定的目录下
            if fileName:
                fileName = fileName.replace(" ","_")
                if not os.path.exists(self._savedir):
                    os.makedirs(self._savedir)
                filePath = os.path.join(self._savedir, fileName)
                fp = open(filePath, 'wb')
                fp.write(part.get_payload(decode=True))
                fp.close()
                attachments.append(filePath)

        return attachments


import re
import datetime as dt

class TimeParser():
    """
    This class is used to handle time transfer of 
    time string,datetime class and timestamp,
    of course, transfer of timedelta and timedetla string 
    """
    @staticmethod
    def transfer_to(obj,to="datetime",string_formats=["%Y-%m-%d","%Y-%m-%d %H:%M:%S","%Y-%m-%d %H:%M:%S.%f"],ostr_format=None):
        """
        automatic detect type of input object and transfer it to indicated type of time
        Argument:
        to: <string|datetime|timestamp>
        """
        time_datetime = None
        if isinstance(obj,str):
            for string_format in string_formats:
                try:
                    time_datetime = dt.datetime.strptime(obj,string_format)
                    break
                except:
                    pass
            if not time_datetime:
                raise Exception("input string {obj} does not match any of formats {string_formats}")
            
        elif isinstance(obj,int) or isinstance(obj,float):
            time_datetime = dt.datetime.fromtimestamp(obj)
        elif isinstance(obj,dt.datetime):
            time_datetime = obj
        
        if to == "string":
            if not ostr_format:
                ostr_format = string_format
            return time_datetime.strftime(ostr_format)
        elif to == "datetime":
            return time_datetime
        elif to == "timestamp":
            return dt.datetime.timestamp(time_datetime)
    
    @staticmethod
    def str_to_timedelta(timestr):
        """
        transfer timedelta string to timedelta class
        """
        time_format = r'((?P<days>\d+) day(s{0,1}), ){0,1}(?P<hours>\d+):(?P<minutes>\d+):(?P<seconds>[\d\.]+)'
        time_re = re.search(time_format,timestr)
        if not time_re:
            raise Exception(f"Unknown time string format {timestr}")
        else:
            days = time_re.group('days') or 0
            days = int(days)
            hours = int(time_re.group('hours'))
            minutes = int(time_re.group('minutes'))
            seconds = float(time_re.group('seconds'))
            
            return dt.timedelta(days=days,hours=hours,minutes=minutes,seconds=seconds)


import logging

#=======================================
#basic_log set the basicConfig of logging and return a logger 
#========================================

def basic_log(name,level=logging.DEBUG,logfile=None,filemode="w",log_format=None):
    """
    Set basic logging config and name a logger
    """
    if not log_format:
        log_format = '[%(asctime)s:%(funcName)s:%(name)s:%(levelname)s] %(message)s'
    if logfile:
        logging.basicConfig(filename=logfile,level=level,
            filemode=filemode,format= log_format
            )
    else:
        logging.basicConfig(level=level,format=log_format)

    logger = logging.getLogger(name)

    return logger

#==========================================
#multi_logger output log to logfile and stdout at the same time
#==========================================
def multi_logger(name,level=logging.DEBUG,logfile='foo.log',mode='w',log_format=None):
    """
    this functions output log to standard output and log file
    """
    if not log_format:
        log_format = '[%(asctime)s:%(funcName)s:%(name)s:%(levelname)s] %(message)s'
        formatter = logging.Formatter(log_format)
    logger = logging.getLogger(name)
    logger.setLevel(level)

    fh = logging.FileHandler(logfile,mode=mode,encoding='utf-8')
    fh.setLevel(level)
    fh.setFormatter(formatter)

    ch = logging.StreamHandler()
    ch.setLevel(level)
    ch.setFormatter(formatter)

    logger.addHandler(fh)
    logger.addHandler(ch)

    return logger



import subprocess as sp
from multiprocessing import Process,Pool
from threading import Thread,Semaphore

def multi_process(assignments):
    """
    assignments is a list of tuples which are
    composed by a function and its arguments(in tuple format)
    this function is synchronized by .join() method
    of Process 
    """
    for func,args,kwargs in assignments:
        p = Process(target=func,args=args,kwargs=kwargs)
        print('subprocess for %s is running...' % func)
        p.start()

    p.join() #block the main process to synchronize the subprocesses
    print('All assignments are completed!')

def process_pool(assignments,number_of_process):
    """
    run multiple processes by specifying number of processes
    each time
    """
    p = Pool(number_of_process)
    for func,args,kwargs in assignments:
        p.apply_async(func,args=args,kwds=kwargs)

    p.close() # no other procee will be added into pool
    p.join() # synchronization
    print('All assignments are completed!')

def run_cmd(cmd):
    """
    run command by using subprocess,
    raise exception when error has happened
    return standard output and standard error
    """
    cp = sp.run(cmd,shell=True,capture_output=True,encoding="utf-8")
    if cp.returncode != 0:
        error = f"""Something wrong has happened when running command [{cmd}]:
         {cp.stderr}"""
        raise Exception(error)
    
    return cp.stdout,cp.stderr

def thread_enhance(thread=None):
    """
    A decorator to provide semaphore and exception handle function
    for mutiple threading 
    ------------------------
    thread: number of thread [integer]
    """
    sema = Semaphore(thread) if thread else None

    def decorator(func):
        def wrapper(*args,**kwargs):

            class TH(Thread):
                def __init__(self):
                    super().__init__()
                    self.error = 0
                    self.msg = "ok"

                def run(self):
                    if sema:
                        sema.acquire()
                    try:
                        func(*args,**kwargs)
                    except Exception as e:
                        pass
                        print("exception happened!")
                        self.error = 1
                        self.msg = e
                    if sema:
                        sema.release()
            return TH()            
        return wrapper
    return decorator


import os
import threading
import time
import re
import urllib
import docx
import requests
import colorama



def make_colors():
    """
    make some colors in command line
    """
    colors = {}
    colorama.init()
    colors['red'] = colorama.Fore.RED
    colors['green'] = colorama.Fore.GREEN
    colors['green_ex'] = colorama.Fore.LIGHTGREEN_EX
    colors['white'] = colorama.Fore.LIGHTWHITE_EX
    colors['magenta_ex'] = colorama.Fore.LIGHTMAGENTA_EX
    colors['magenta'] = colorama.Fore.MAGENTA
    colors['cyan'] = colorama.Fore.CYAN
    colors['cyan_ex'] = colorama.Fore.LIGHTCYAN_EX
    colors['yellow'] = colorama.Fore.YELLOW
    colors['bright'] = colorama.Style.BRIGHT
    colors['back'] = colorama.Style.RESET_ALL
    return colors
    
#=======================================================
#TitleParser is used to get columns by file's title name
#========================================================
class TitleParser():
    """
    Reading in a file title and than getting indicated column element.
    Title names are all CaseIgnore
    """
    
    def __init__(self,title,split="\t"):
        self.split = split
        self.title_list = [i.lower() for i in title.strip().split(self.split)]

    def get_field(self,line_list,colname,check=True):
        """
        Reading in a list and returning the element with the 
        index which colname in title's list
        """
        if check:
            if len(self.title_list) != len(line_list):
                raise Exception("Title length differs with line!")
        try:
            idx = self.title_list.index(str(colname).lower())
        except:
            print('{colname} not in title！'.format(colname=colname))
            return None

        return line_list[idx]

    def have_title(self,colname):
        """
        Judging whether a colname is in title
        """
        if str(colname).lower() in self.title_list:
            return True
        return False
    
    def get_idx(self,colname):
        """
        return index of columns by their name in title list
        """
        idx = self.title_list.index(str(colname).lower())

        return idx

#========================================
#Class DocxApi help to operate Words easier 
#like add paragraph,picture or table to the
# place you want to insert etc.
#========================================
class DocxApi():
    """
    docx api implement adding,replacing and deletion operation
    """
    def __init__(self,docx_file=None):
        self.doc = docx.Document(docx_file) if docx_file else docx.Document()

    def align_center(self,paragraph):
        """
        align a paragraph center
        """
        paragraph.alignment = docx.enum.text.WD_ALIGN_PARAGRAPH.CENTER
        return paragraph
    
    def set_font(self,en_style='Times New Roman',zh_style='微软雅黑'):
        """
        set global font 
        """
        self.doc.styles['Normal'].font.name = en_style
        self.doc.styles['Normal']._element.rPr.rFonts.set(docx.oxml.ns.qn('w:eastAsia'),zh_style)

    def set_para_font_size(self,paragraph,size=10.5):
        """
        set paragraph font size
        """
        for run in paragraph.runs:
            run.font.size = docx.shared.Pt(size)
        
        return paragraph

    def set_tab_font_size(self,table,size=10.5):
        """
        set table font size
        """
        for row in table.rows:
            for cell in row.cells:
                for pargraph in cell.paragraphs:
                    self.set_para_font_size(pargraph,size=size)
        return table

    def add_picture(self,pic_path,width=5.0,height=None):
        """
        In order to move picture to some place you want.
        This function will build a new paragprah and then 
        use the 'run.add_picture()' method to add picture. After that,
        you can move it like moving a paragraph to insert to docx or
        justify styles.
        
        Width 5.0 is the proper picture width in Test,you can change it
        to match your need.
        """
        paragraph = self.doc.add_paragraph()
        run = paragraph.add_run()
        if height:
            height = docx.shared.Inches(height)
            
        run.add_picture(pic_path,
                        width=docx.shared.Inches(width),
                        height=height)

            
        return paragraph

    def add_page_break(self):
        """
        start from new page
        """
        self.doc.add_page_break()

    def delete_tab_para(self,table):
        """
        delete table or paragraph
        """
        table._element.getparent().remove(table._element)
        print('remove complete!')

    def add_paras(self,content):
        """
        add a new paragraph
        """
        try:
            return self.doc.add_paragraph(content.decode('utf-8'))
        except:
            return self.doc.add_paragraph(content)
            
    def add_table(self,table_list):
        """
        add a new table
        table_list : [[1,2,3],[4,5,6],...]
        """
        nrow = len(table_list)
        ncol = len(table_list[0])
        print(nrow,ncol)
        table = self.doc.add_table(0,ncol)
        for row in table_list:
            row_cells = table.add_row().cells
            for idx,cell in enumerate(row):
                try:
                    row_cells[idx].text = cell.decode('utf-8')
                except:
                    row_cells[idx].text = cell
                    # print("{cell} of {row} can\'t be decode correctly!".format(cell=cell,row=row))
                    # print(cell,row)
                    # exit(1)
        return table

    def move_obj(self,obj,tag):
        """
        move the obj to the next of tag
        """
        if isinstance(obj,docx.table.Table):
            obj = obj._tbl
        elif isinstance(obj,docx.text.paragraph.Paragraph):
            obj = obj._p
        else:
            print('UnKnown type of input object!')
            exit(1)
        for paragraph in self.doc.paragraphs:
            if paragraph.text.startswith(tag.decode('utf-8')):
                paragraph._p.addnext(obj)
                print('add complete!')
    
    def save(self,ofile=None):
        if not ofile:
            ofile = 'test.docx'
        self.doc.save(ofile)

    def replace_para(self,paragraph,new_content):
        """
        clear paragraph and add new content
        """
        paragraph.clear()
        paragraph.add_run(new_content)
    
        return paragraph
    
    def get_pt_by_size(self,size):
        """
        transformation chinese size 号 to pt
        """
        pt_hao_trans_dict = {
                            '八号':5.0,
                            '七号':5.5,
                            '小六':6.5,
                            '六号':7.5,
                            '小五':9.0,
                            '五号':10.5,
                            '小四':12.0,
                            '四号':14.0,
                            '小三':13.0,
                            '三号':16.0,
                            '小二':18.0,
                            '二号':22.0,
                            '小一':24.0,
                            '一号':26.0,
                            '小初':36.0,
                            '初号':42.0,
                            }
        if size in pt_hao_trans_dict:
            # return docx.shared.Pt(pt_hao_trans_dict[size])
            return pt_hao_trans_dict[size]
        else:
            print('Unknown input size,size should be in {keys}'.format(keys=list(pt_hao_trans_dict)))
            return None
        
#################################################################
# download process bar
# 
#################################################################

def view_bar(num, total):  #显示进度条
    rate = num/total
    rate_num = int(rate * 100)
    number=int(50*rate)
    r = '\r[%s%s]%d%%' % ("#"*number, " "*(50-number), rate_num, )
    print("\r {}".format(r),end=" ")   #\r回到行的开头
    
class Getfile():  #下载文件
    def __init__(self,url):
        self.url=url
        #self.filename=filename
        self.re=requests.head(url,allow_redirects=True)  #运行head方法时重定向
    def getsize(self):
        try:
            self.file_total=int(self.re.headers['Content-Length']) #获取下载文件大小    
            return self.file_total
        except:
            print('无法获取下载文件大小')
            exit()
    def getfilename(self):  #获取默认下载文件名
        filename=''
        if 'Content-Disposition' in self.re.headers:
            n=self.re.headers.get('Content-Disposition').split('name=')[1]
            filename=urllib.parse.unquote(n,encoding='utf8')
        elif os.path.splitext(self.re.url)[1]!='':
            filename=os.path.basename(self.re.url)
        return filename
    def downfile(self,filename):  #下载文件
        self.r = requests.get(self.url,stream=True)
        with open(filename, "wb") as f:
            for chunk in self.r.iter_content(chunk_size=1024): #边下载边存硬盘
                if chunk:
                    f.write(chunk)
        time.sleep(1)
        #print ("\n下载完成")
    def downprogress(self,filename):
        self.filename=filename
        self.file_size=0
        self.file_total=self.getsize()
        while self.file_size<self.file_total:  #获取当前下载进度
            time.sleep(1)
            if os.path.exists(self.filename):
                self.down_rate=(os.path.getsize(self.filename)-self.file_size)/1024/1024
                self.down_time=(self.file_total-self.file_size)/1024/1024/self.down_rate
                print (" "+str('%.2f' %self.down_rate+"MB/s"),end="")
                self.file_size=os.path.getsize(self.filename)
                print (" "+str(int(self.down_time))+"s",end="")
                print (" "+str('%.2f' %(self.file_size/1024/1024))+"MB",end="")
                view_bar(self.file_size, self.file_total)
            
def progress_bar(url,ofile=None):
    file1=Getfile(url)
    file_total=file1.getsize()
    if not ofile:
        filename=file1.getfilename()
    else:
        filename = ofile
    print ("下载文件大小："+str('%.2f' % (file_total/1024/1024))+"MB")
    print ("开始下载文件:"+filename)
    t1 = threading.Thread(target=file1.downfile,args=(filename,))
    t1.start()
    file1.downprogress(filename)

def datasize_convert(size,convert_unit,return_type="float"):
    """
    convert unit between B/KB/MB/GB/TB/PB 
    ----------
    Argument:
    return_type <float|int|str> if str is selected, unit will be added to digitals
    """
    size = str(size)
    unit_list = ["B","K","M","G","T",'P']
    unit_re = re.search(r"^([\d\.]+)([A-Z]){0,1}",size.upper())
    if unit_re:
        size_value = unit_re.group(1)
        size_unit = unit_re.group(2) or "B"
        # print(size_value,size_unit)
        target_unit = convert_unit.upper().split("B")[0] or "B"
        convert_size = float(size_value) / 1024 ** (unit_list.index(target_unit) - unit_list.index(size_unit))
        print(f"transfer {size_value}{size_unit} to {convert_size}{target_unit}")
        if return_type == "float":
            return convert_size
        elif return_type == "int":
            return int(convert_size)
        elif return_type == "str":
            return str(convert_size) + target_unit
        else:
            raise Exception(f"Unknown return_type {return_type}! select from <float|int|str>")

    raise Exception(f"Convert Error for {size}!")

if __name__ == '__main__':
    url = "https://www.cyberpunk.net/uploads/wallpapers/2560x1600/CP77-KV-zh-cn.jpg"  
    #url可指定下载的文件的url; name可带路径
 
 
    #url = input("输入网址:")
    #while not re.match('^(https?|ftp)://.+$',url):
    #    url = input("网址格式错误，请重新输入:")
    file1=Getfile(url)
    file_total=file1.getsize()
    filename=file1.getfilename()
    if filename=='':
        filename=input('无法获取下载文件名,请自行输入:')
    print ("下载的文件为："+str('%.2f' % (file_total/1024/1024))+"MB")
    print ("开始下载文件:"+filename)
    t1 = threading.Thread(target=file1.downfile,args=(filename,))
    t1.start()
    file1.downprogress(filename)




#!/usr/bin/env python
# -*- coding=utf-8 -*-

__author__ = "cm.Rpy"

import sys
import re
import signal
import click
import colorama

#solve [Error 32] Broken pipe
signal.signal(signal.SIGPIPE,signal.SIG_DFL)


class DataExtracter:
    def __init__(self,**kwargs):
        self.indata = kwargs.get("filename")
        self.colnames = kwargs.get("column")
        self.allmark = kwargs.get("title")
        self.exceptmark = kwargs.get("except")
        self.ignoremark = kwargs.get("ignore")
        self.split = kwargs.get("split")
        
    def first5line_extract(self):
        first5line = []
        i = 0
        if self.ignoremark != None:
            for line in self.indata:
                if not line.strip().startswith(self.ignoremark) and i < 5:
                    if re.search("^$",line.strip()):
                        break
                    else:
                        first5line.append(line.strip())
                        i += 1
                elif i >= 5:
                    break
        else:
            while i < 5:
                line = self.indata.readline()
                if re.search("^$",line.strip()):
                    break
                else:
                    first5line.append(line.strip())
                    i += 1
        return(first5line)

    def show_summary(self,):
        first5line = self.first5line_extract()
        title = first5line.pop(0).split(self.split)
        colstrings = []
        #print(len(title))
        for i in range(len(title)):
            eachcol_list = []

            for ele in first5line:
                try:
                    eachcol_list.append(ele.split(self.split)[i])
                except:
                    eachcol_list.append("无对应值")
            
            eachstring = " ".join(eachcol_list)
            each_str_len = len(eachstring)
            if each_str_len > 50:
                eachstring = eachstring[0:51] + "..."
            else:
                eachstring = eachstring + "..."
            colstrings.append(eachstring)
        
        longest_value = max([len(i) for i in title])
        for j in range(len(title)):
            print("{bright}{color1}{order} {color2}{colname: >{value}}\t{color3}{content}{back}".format(
                color1 = colorama.Fore.YELLOW,color2 = colorama.Fore.RED,color3 = colorama.Fore.GREEN,
                bright=colorama.Style.BRIGHT,order = j+1, colname = title[j], content = colstrings[j],
                value = longest_value,back=colorama.Style.RESET_ALL
            )) 
    
    def extract_columns(self):
        
        seps=[re.split(":|-",i) if re.search(":|-",i) else [i] for i in self.colnames.split(",") ]
        #print(seps)
        first5line = self.first5line_extract()
        title = first5line[0].split(self.split)
        col_num_mark = [] #all column numbers colnames indicated
        for ele in seps:
            if len(ele) == 1 :
                if ele[0] not in title:
                    raise Exception(f"{ele[0]} not in title {title} or wrong split indicated, please check!")
                linenum = title.index(ele[0])
                col_num_mark.append(linenum)
            elif len(ele) == 2 :
                startnum = title.index(ele[0])
                endnum = title.index(ele[1])
                for num in range(startnum,endnum+1):
                    col_num_mark.append(num)
            assert len(ele) < 3, " May your colnames parameter was wrong, please check "
        if self.exceptmark:
            col_num_mark = [n for n in range(len(title)) if n not in col_num_mark ]
        return(col_num_mark)

    def column_print(self):
        col_num_mark = self.extract_columns()
        for line in self.indata:
            if self.ignoremark !=None:
                if line.strip().startswith(self.ignoremark):
                    pass
                else:
                    selected_cols = []
                    for i in col_num_mark:
                        selected_cols.append(line.strip().split(self.split)[i])
                    print(f"{self.split}".join([str(n) for n in selected_cols]))
                        
            else:
                selected_cols = []
                for i in col_num_mark:
                    selected_cols.append(line.strip().split(self.split)[i])
                print(f"{self.split}".join([str(n) for n in selected_cols])) 

@click.command()
@click.argument("filename",default=sys.stdin,type=click.File('r'))
@click.option("--title","-t",is_flag=True,help="list title information of input file")
@click.option("--split","-s",help="seperator to split each column default[\\t]",default="\t")
@click.option("--column","-c",help="columns to extract by their name,comma split")
@click.option("--except",'-e',is_flag=True,help="extract columns except indicated columns")
@click.option("--ignore",'-i',help="ignore lines start with indicated string")
def workflow(**kwargs):
    """\b
    Example:
        function1:show all colnames:
        getcol.py -t <Filename>
        cat file | getcol -t
        
        function2:select indicated colname:
        getcol.py -c colname1:colname2 <Filename>
        getcol.py -e -c colname1:colname2 <Filename>
        getcol.py -e -c colname1:colname2,colname3 --ignore "#" <Filename>
    """
    df = DataExtracter(**kwargs)
    #print(df.allmark)
    #print(df.exceptmark)
    #print(df.colnames)
    #df.show_summary()
    if df.allmark and df.exceptmark or df.allmark and df.colnames :
        raise SyntaxError("{}-a parameter will be not used with other parameters{}".format(
            colorama.Fore.RED,colorama.Style.RESET_ALL
        ))
    elif df.allmark:
        df.show_summary()
    elif df.colnames:
        df.column_print()


if __name__=="__main__":
    #print(__author__)
    colorama.init()
    #print(args)
    #print(df.first5line_extract()
    #df.column_print()
    workflow()
    



import setuptools
from setuptools import setup

with open('README.md','r') as fh:
    long_description = fh.read()

setup(
    name = "novotools",
    version = "0.0.6",
    packages = setuptools.find_packages(),
    url = "https://github.com/pypa/novotools",
    license = 'MIT',
    author = "chenming",
    author_email = "chenming@novogene.com",
    description = "a common tools to facilitate your coding",
    long_description = long_description,
    long_description_content_type = "text/markdown",
    install_requires = ["python-docx","requests","colorama","openpyxl","click"],
    classifiers = [
        "Development Status :: 2 - Pre-Alpha",
        "Programming Language :: Python :: 3.8"],
    python_requires = ">=3.8",
    scripts = ["novotools/tools/excel2text.py",
               "novotools/tools/text2excel.py",
               "novotools/tools/getcol.py"],
)


#!/usr/bin/env python

# -*- coding=utf-8 -*-

#Extracting indicated sheet to plain text

import argparse
import datetime
from openpyxl import load_workbook


time = datetime.datetime.now().strftime('%Y-%m-%d')

def excel_to_txt(args):
    excel = args.get('excel')
    sheet_names = args.get('sheet_name')
    sheet_numbers = args.get('sheet_number') 
    book = load_workbook(excel)
    if args.get('show_sheets'):
        print('All sheets in the excel lists blow:\n')
        for idx,sheetname in enumerate(book.sheetnames):
            print('[{idx}] {sheetname}'.format(idx=idx,sheetname=sheetname))
    else:
        sheet_list = []
        if sheet_numbers:
            sheet_list += [book.sheetnames[i] for i in sheet_numbers]
        if sheet_names and (sheet_names not in sheet_list):
            sheet_list += sheet_names
        if (not sheet_numbers) and (not sheet_names):
            sheet_list = book.sheetnames

        for sheet in sheet_list:
            ofile = sheet + '.' + time + '.xls'
            print('Writing %s to %s' % (sheet,ofile))
            with open(ofile,'w') as odata:
                for row in book[sheet]:
                    row_list = [cell.value for cell in row]
                    row_list = [args.get('handle_NA') if i is None else str(i) for i in row_list]
                    odata.write("\t".join(row_list) + "\n")
        print('Writing Completion.') 

if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        formatter_class=argparse.RawDescriptionHelpFormatter,
        epilog='''
        Example:
        
            excel_to_txt.py -e excel  # extract all sheets 
            excel_to_txt.py -e excel --sheet_number 0 1 2
            excel_to_txt.py -e excel --sheet_name "sheet1" "sheetx"
        '''
    )
    parser.add_argument('--excel','-e',required=True,help='Excel Name')
    parser.add_argument('--sheet_number','-num',nargs="*",default=[],type=int,help='The index of sheet number zero-based')
    parser.add_argument('--sheet_name','-name',nargs="*",default=[],help='One or more sheet names')
    parser.add_argument('--handle_NA','-NA',default='',
        help='what shold be used to replace None objects[default:""]')
    parser.add_argument('--show_sheets','-show',action='store_true',help='Show the sheets of excel instead of extracting them')
    args = parser.parse_args()
    excel_to_txt(vars(args))
